import os
import sys
import argparse
from shexer.shaper import Shaper
from shexer.consts import TURTLE, MIXED_INSTANCES

def run(base_input_dir,
        out_path,
        namespaces):

    _files = [os.path.join(dp, f) for dp, dn, filenames in
                os.walk(base_input_dir) for f in filenames]
    in_files = [os.path.join(dp, f) for dp, dn, filenames in
                os.walk(base_input_dir) for f in filenames
                if os.path.splitext(f)[1] == '.gz']
    shaper = Shaper(graph_list_of_files_input=in_files,
                    all_classes_mode=True,
                    input_format=TURTLE,
                    namespaces_dict=namespaces,
                    disable_exact_cardinality=True,
                    detect_minimal_iri=True,
                    compression_mode="gz",
                    instances_report_mode=MIXED_INSTANCES)

    # Verbose active mode, so one can check in which stage is the execution, some raw numbers
    # about shapes and instances computed, and also execution times

    # This acceptance_threshold filters any information observed in less than 5% of the
    # instances of any class.
    if out_path == "__STD_OUT__":
       shaper.shex_graph(string_output=True,
                         verbose=True,
                         acceptance_threshold=0.05)
    else:
       shaper.shex_graph(output_file=out_path,
                         verbose=True,
                         acceptance_threshold=0.05)

    print("Done!")

if __name__ == "__main__":
    ############### CONFIGURATION ###############

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', help='Specify the input directory')
    parser.add_argument('-o', '--output', help='Specify the output file')
    args = parser.parse_args()
    if args.input:
       base_input_dir = args.input
    else:
       print('Specify the input directory.', file=sys.stderr)
       sys.exit(-1)

    if args.output:
       out_path = args.output
    else:
       out_path = "__STD_OUT__"

    # Directory with the wikipathways dump (content unzipped). the process will recursively look
    # for any ttl file in this folder or any of this subfolders, and it will merge it in a single
    # graph
#    base_input_dir = r"/opt/rdf-import/knapsack"

    # output shex file
#    out_path = r"shaper_knapsack_allclass.shex"

    # namespace-prefix pair to be used in the results
    namespaces_dict = {"http://purl.org/dc/terms/": "dc",
                       "http://rdfs.org/ns/void#": "void",
                       "http://www.w3.org/2001/XMLSchema#": "xsd",
                       "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
                       "http://www.w3.org/ns/dcat#": "dcat",
                       "http://xmlns.com/foaf/0.1/": "foaf",
                       "http://www.w3.org/2002/07/owl#": "owl",
                       "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
                       "http://www.w3.org/2004/02/skos/core#": "skos",
                       }
    ############### EXECUTION ###############

    run(base_input_dir=base_input_dir,
        out_path=out_path,
        namespaces=namespaces_dict)


from shexer.core.profiling.class_profiler import RDF_TYPE_STR

from shexer.model.property import Property
from shexer.utils.uri import remove_corners
from shexer.utils.shapes import prefixize_shape_name_if_possible
from shexer.io.shex.formater.consts import SPACES_LEVEL_INDENTATION
from shexer.io.wikidata import wikidata_annotation
from shexer.io.file import read_file
from shexer.consts import RATIO_INSTANCES, ABSOLUTE_INSTANCES, MIXED_INSTANCES

from wlighter import SHEXC_FORMAT

_MODES_REPORT_INSTANCES = [ABSOLUTE_INSTANCES, MIXED_INSTANCES]

class ShexSerializer(object):

    def __init__(self, target_file, shapes_list, namespaces_dict=None, string_return=False,
                 instantiation_property_str=RDF_TYPE_STR, disable_comments=False, wikidata_annotation=False,
                 instances_report_mode=RATIO_INSTANCES, detect_minimal_iri=False):
        self._target_file = target_file
        self._shapes_list = shapes_list
        self._lines_buffer = []
        self._namespaces_dict = namespaces_dict if namespaces_dict is not None else {}
        self._string_return = string_return
        self._instantiation_property_str = self._decide_instantiation_property(instantiation_property_str)
        self._disable_comments = disable_comments
        self._wikidata_annotation = wikidata_annotation
        self._instances_report_mode = instances_report_mode
        self._detect_minimal_iri = detect_minimal_iri

        self._string_result = ""

    def serialize_shapes(self):

        self._reset_target_file()
        self._serialize_namespaces()
        for a_shape in self._shapes_list:
            self._serialize_shape(a_shape)
        self._flush()
        if self._wikidata_annotation:
            self._annotate_wikidata_ids_in_result()
        if self._string_return:
            return self._string_result

    @staticmethod
    def _decide_instantiation_property(instantiation_property_str):
        if instantiation_property_str == None:
            return RDF_TYPE_STR
        if type(instantiation_property_str) == Property:
            return str(instantiation_property_str)
        if type(instantiation_property_str) == str:
            return remove_corners(a_uri=instantiation_property_str,
                                  raise_error_if_no_corners=False)
        raise ValueError("Unrecognized param type to define instantiation property")

    def _annotate_wikidata_ids_in_result(self):
        self._string_result =  wikidata_annotation(raw_input=self._get_raw_input_for_wikidata_annotation(),
                                                   string_return=self._string_return,
                                                   out_file=self._target_file,
                                                   format=SHEXC_FORMAT,
                                                   rdfs_comments=True)

    def _get_raw_input_for_wikidata_annotation(self):
        if self._string_return:
            return self._string_result
        return read_file(self._target_file)


    def _serialize_namespaces(self):
        for a_namespace in self._namespaces_dict:
            self._write_line(self._prefix_line(a_namespace), 0)
        self._write_line("", 0)

    def _prefix_line(self, namespace_key):
        return "PREFIX " + self._namespaces_dict[namespace_key] + ": <" + namespace_key + ">"

    def _serialize_empty_namespace(self):
        self._write_line("PREFIX : <http://weso.es/shapes/>")

    def _serialize_shape(self, a_shape):
        self._serialize_shape_name(a_shape)
        self._serialize_opening_of_rules()
        self._serialize_shape_rules(a_shape)
        self._serialize_closure_of_rule()
        self._serialize_shape_gap()

    def _flush(self):
        self._write_lines_buffer()

    def _write_line(self, a_line, indent_level=0):
        self._lines_buffer.append(self._indentation_spaces(indent_level) + a_line + "\n")
        if len(self._lines_buffer) >= 5000:
            self._write_lines_buffer()
            self._lines_buffer = []

    def _reset_target_file(self):
        if self._string_return:
            return
        with open(self._target_file, "w") as out_stream:
            out_stream.write("")  # Is this necessary? maybe enough to open it in 'w' mode?

    def _write_lines_buffer(self):
        if self._string_return:
            self._string_result += "".join(self._lines_buffer)
        else:
            with open(self._target_file, "a") as out_stream:
                for a_line in self._lines_buffer:
                    out_stream.write(a_line)

    def _indentation_spaces(self, indent_level):
        result = ""
        for i in range(0, indent_level):
            result += SPACES_LEVEL_INDENTATION
        return result

    def _serialize_shape_rules(self, a_shape):
        if a_shape.n_statements == 0:
            return
        statements = [a_statement for a_statement in a_shape.yield_statements()]
        for i in range(0, len(statements) - 1):
            for line_indent_tuple in statements[i]. \
                    get_tuples_to_serialize_line_indent_level(is_last_statement_of_shape=False,
                                                              namespaces_dict=self._namespaces_dict):
                self._write_line(a_line=line_indent_tuple[0],
                                 indent_level=line_indent_tuple[1])
        for line_indent_tuple in statements[len(statements) - 1]. \
                get_tuples_to_serialize_line_indent_level(is_last_statement_of_shape=True,
                                                          namespaces_dict=self._namespaces_dict):
            self._write_line(a_line=line_indent_tuple[0],
                             indent_level=line_indent_tuple[1])

    def _serialize_shape_name(self, a_shape):
        self._write_line(

            prefixize_shape_name_if_possible(a_shape_name=a_shape.name,
                                             namespaces_prefix_dict=self._namespaces_dict) +
            self._minimal_iri(a_shape=a_shape) +
            self._instance_count(a_shape)
        )

    def _minimal_iri(self, a_shape):
        if not self._detect_minimal_iri or a_shape.iri_pattern is None:
            return ""
        return "  [<{}>~]  AND".format(a_shape.iri_pattern)

    def _instance_count(self, a_shape):
        return "   # {} instance{}".format(a_shape.n_instances,
                                           "" if a_shape.n_instances == 1 else "s") \
            if self._instances_report_mode in _MODES_REPORT_INSTANCES and \
                     not self._disable_comments \
            else ""

    def _serialize_opening_of_rules(self):
        self._write_line("{")

    def _serialize_closure_of_rule(self):
        self._write_line("}")

    def _serialize_shape_gap(self):
        self._write_line("")
        self._write_line("")



import os
import argparse
import sys
from shexer.shaper import Shaper
from shexer.consts import TURTLE, MIXED_INSTANCES

def run(base_input_dir,
        out_path,
        namespaces):

#     in_files = [os.path.join(dp, f) for dp, dn, filenames in
#                 os.walk(base_input_dir) for f in filenames
# #                if os.path.splitext(f)[1] == '.ttl']
#                 if os.path.splitext(f)[1] == '.gz']

    shaper = Shaper(graph_list_of_files_input=["galen.ttl.gz"],
                    all_classes_mode=True,
                    input_format=TURTLE,
                    namespaces_dict=namespaces,
                    disable_exact_cardinality=True,
                    detect_minimal_iri=True,
                    compression_mode="gz",
                    instances_report_mode=MIXED_INSTANCES)

    # Verbose active mode, so one can check in which stage is the execution, some raw numbers
    # about shapes and instances computed, and also execution times

    # This acceptance_threshold filters any information observed in less than 5% of the
    # instances of any class.
    shaper.shex_graph(output_file=out_path,
                      verbose=True,
                      acceptance_threshold=0.05)

    print("Done!")

if __name__ == "__main__":
    ############### CONFIGURATION ###############

    # parser = argparse.ArgumentParser()
    # parser.add_argument('-i', '--input', help='Specify the input directory')
    # parser.add_argument('-o', '--output', help='Specify the output file')
    # args = parser.parse_args()
    # if args.input:
    #    base_input_dir = args.input
    # else:
    #    print('Specify the input directory.', file=sys.stderr)
    #    sys.exit(-1)
    # if args.output:
    #    out_path = args.output
    # else:
    #    out_path = "__STD_OUT__"
    # # Directory with the wikipathways dump (content unzipped). the process will recursively look
    # # for any ttl file in this folder or any of this subfolders, and it will merge it in a single
    # # graph
    #
    # # output shex file
    #
    # # namespace-prefix pair to be used in the results
    namespaces_dict = {"http://purl.org/dc/terms/": "dc",
                       "http://rdfs.org/ns/void#": "void",
                       "http://www.w3.org/2001/XMLSchema#": "xsd",
                       "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
                       "http://purl.org/pav/": "pav",
                       "http://www.w3.org/ns/dcat#": "dcat",
                       "http://xmlns.com/foaf/0.1/": "foaf",
                       "http://www.w3.org/2002/07/owl#": "owl",
                       "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
                       "http://www.w3.org/2004/02/skos/core#": "skos",
                       }
    ############### EXECUTION ###############

    run(base_input_dir="",
        out_path="here.shex",
        namespaces=namespaces_dict)



"""
@Author: Rubén Luque
@date: 13/11/2023
@email: luqueruben@uniovi.es
"""
import random
import time

player_name = ""
health = 100
protector = ""
coins = 0
animals = ["Lobo", "Oso", "Tigre", "Rottweiler", "Akita Inu"]
treasures = ["monedas de oro", "latas de cobre", "baratijas", "kgs de arroz"]

"""
Juego sencillo con 1 sola condición de victoria: Encontrar 5 monedas de oro y salud > 0.
Y una condición de derrota: Tu salud es 0 o menor.
"""


def start_adventure():
    global player_name
    print("Bienvenido a la Aventura en el Bosque Encantado.")
    player_name = input("¿Serías tan amable de indicarme tu nombre?: ").capitalize()
    print(f"Hola {player_name}, comenzamos...\n")

    while True:
        choice = input(
            "Ves un camino que se divide en dos: ¿Vas a la izquierda "
            "o a la derecha? (I/D) ").lower()
        if choice in ["izquierda", "i"]:
            # Genero número aleatorio para el animal
            animal_id = random.randrange(0, len(animals))  # No llega al fin, se queda en uno menos.
            encounter_animal(animals[animal_id])
        elif choice in ["derecha", "d"]:
            treasure_id = random.randrange(0, len(treasures))  # No llega al fin, se queda en uno
            find_treasure(treasures[treasure_id])
        else:
            print("Opción no válida. Elige 'I' o 'D'.")

        if check_game_over():
            print(f"FIN DEL JUEGO!\n"
                  f"Lo siento {player_name}, has perdido. "
                  f"Salud: {health}. Monedas: {coins}")
            break
        if check_victory():
            print(f"FIN DEL JUEGO!\n"
                  f"Enhorabuena {player_name}!! Has ganado la partida. "
                  f"Salud: {health}. Monedas: {coins}")
            break


def encounter_animal(animal_name):
    global protector
    print(f"¡Te encuentras con un {animal_name}!. Parece hambriento...")
    action = input("¿Qué haces? (correr(C)/pelear(P)/alimentar(A)): ").lower()
    if action in ["correr", "c"]:
        print(f"Corres rápido y escapas del {animal_name}. ¡Seguro por ahora!")
    elif action in ["pelear", "p"]:
        print(f"Decides enfrentarte al {animal_name}.")
        fight_animal(animal_name)
    elif action in ["alimentar", "a"]:
        print(f"Le das tu comida al {animal_name}. "
              f"Se convierte en tu amigo y te acompaña en tu aventura.")
        protector = animal_name
    else:
        print("Opción no válida.")


def fight_animal(animal_name):
    global health
    # global nombre  # Si solo leemos su valor, no hace falta definirla aquí como global
    strength = int(input("¿Cuánto mides tu fuerza en una escala de 1 a 10?: "))
    protector_help = ""

    while strength < 1 or strength > 10:
        strength = int(input("ERROR: Te he dicho una escala de 1 a 10! Introduce otra vez: "))
    if len(protector) > 0:
        protector_help = f"Tu {protector} te ayuda atacando al {animal_name}\n"
        damage_reduction_factor = 0.5  # Si tienes protector, se reduce el daño a la mitad
    else:
        damage_reduction_factor = 1  # Sin protector, el daño es completo
    if strength > 5:
        damage = random.randint(0, 50)
        health -= damage * damage_reduction_factor
        print(f"Tu fuerza es suficiente. Vences al {animal_name} en la pelea. {protector_help}Salud actual: {health}")
    else:
        damage = random.randint(20, 100)
        health -= damage * damage_reduction_factor
        print(f"No eres lo suficientemente fuerte. Lo siento {player_name}, {animal_name} te vence."
              f"{protector_help}Salud actual: {health}")


def find_treasure(treasure_name):
    global coins
    print("Encuentras un tesoro escondido detrás de un árbol...\n"
          "Vamos a ver que hay....")
    quantity_found = 0
    for i in range(random.randrange(0, 6)):
        quantity_found += 1
        print(f"Descubriendo el tesoro, espera... ")
        time.sleep(1)
    if treasure_name == "monedas de oro":
        coins += quantity_found
    print(f"Has encontrado {quantity_found} {treasure_name}. ¡Felicidades {player_name}!."
          f" Tienes: {coins} monedas de oro.")


def check_game_over():
    return health <= 0


def check_victory():
    return health > 0 and coins >= 5


start_adventure()
